@@ -38,7 +38,10 @@ use warnings;
 use App::CELL qw( $CELL $log $meta $site );
 use App::Dochazka::Common::Model;
 use App::Dochazka::REST::ConnBank qw( $dbix_conn );
-use App::Dochazka::REST::Model::Employee;
+use App::Dochazka::REST::Model::Employee; 
+use App::Dochazka::REST::Model::Interval qw(
+    fetch_intervals_by_eid_and_tsrange_inclusive
+);
 use App::Dochazka::REST::Model::Shared qw(
     canonicalize_tsrange
     split_tsrange
@@ -118,7 +121,15 @@ BEGIN {
 
         # return an appropriate throw-away value
         return;
-    }
+    };
+
+    *{ 'TO_JSON' } = sub {
+        my $self = shift;
+        my $unblessed_copy;
+        map { $unblessed_copy->{$_} = $self->{$_}; } keys %attr;
+        return $unblessed_copy;
+    };
+
 }
 
 my %dow_to_num = (
@@ -214,12 +225,15 @@ sub _vet_date_spec {
     $log->debug( "Entering " . __PACKAGE__ . "::_vet_date_spec to enforce date specification policy" );
 
     if ( defined( $ARGS{date_list} ) and defined( $ARGS{tsrange} ) ) {
+        $log->debug( "date_spec is NOT OK" );
         return $CELL->status_not_ok;
     }
     if ( ! defined( $ARGS{date_list} ) and ! defined( $ARGS{tsrange} ) ) {
+        $log->debug( "date_spec is NOT OK" );
         return $CELL->status_not_ok;
     }
     $self->{'vetted'}->{'date_spec'} = 1;
+    $log->debug( "date_spec is OK" );
     return $CELL->status_ok;
 }
 
@@ -254,6 +268,9 @@ sub _vet_date_list {
         date_list => { type => ARRAYREF|UNDEF },
     } );
     $log->debug( "Entering " . __PACKAGE__ . "::_vet_date_list to vet/populate the date_list property" );
+    if ( $ARGS{'date_list'} ) {
+        $log->debug( "Date list is " . Dumper $ARGS{'date_list'} );
+    }
 
     die "GOPHFQQ! tsrange property must not be populated in _vet_date_list()" if $self->tsrange;
 
@@ -332,7 +349,8 @@ attributes. Returns a status object.
 sub _vet_tsrange {
     my $self = shift;
     my %ARGS = @_;
-    $log->debug( "Entering " . __PACKAGE__ . "::_vet_tsrange to vet the tsrange $ARGS{tsrange}" );
+    $log->debug( "Entering " . __PACKAGE__ . "::_vet_tsrange to vet the tsrange " . 
+                 ( defined( $ARGS{tsrange} ) ? $ARGS{tsrange} : "(undef)" ) );
 
     die "YAHOOEY! No DBIx::Connector in object" unless $self->context->{dbix_conn};
 
@@ -644,9 +662,10 @@ sub new {
         date_list => { type => ARRAYREF, optional => 1 },
         long_desc => { type => SCALAR|UNDEF, optional => 1 },
         remark => { type => SCALAR|UNDEF, optional => 1 },
-        clobber => { type => BOOLEAN, default => 0 },
-        dry_run => { type => BOOLEAN, default => 0 },
+        clobber => { default => 0 },
+        dry_run => { default => 0 },
     } );
+    $log->debug( "Entering " . __PACKAGE__ . "::new" );
 
     my ( $self, $status );
     # (re-)initialize $self
@@ -659,6 +678,9 @@ sub new {
     die "AGHOOPOWDD@! No tiid in Fillup object!" unless $self->tiid;
 
     map {
+        if ( ref( $ARGS{$_} ) eq 'JSON::PP::Boolean' ) {
+            $ARGS{$_} = $ARGS{$_} ? 1 : 0;
+        }
         $self->$_( $ARGS{$_} ) if defined( $ARGS{$_} );
     } qw( long_desc remark clobber dry_run );
 
@@ -697,10 +719,9 @@ INSERTed into the intervals table.
 
 sub commit {
     my $self = shift;
-    $log->debug( "Entering " . __PACKAGE__ . "::commit with dry_run " . 
-        $self->dry_run ? "TRUE" : "FALSE" );
+    $log->debug( "Entering " . __PACKAGE__ . "::commit with dry_run " . ( $self->dry_run ? "TRUE" : "FALSE" ) );
 
-    my ( $status, @result_set, @fail_set, $count, $ok_count, $not_ok_count );
+    my ( $status, @result_set, @fail_set, $count, $ok_count, $not_ok_count, @deleted_set );
 
     $ok_count = 0;
     $not_ok_count = 0;
@@ -714,6 +735,12 @@ sub commit {
 
         # For each tempintvl object, make a corresponding interval object
         TEMPINTVL_LOOP: foreach my $tempintvl ( @$tempintvls ) {
+
+            # if clobber is true, we have to check each interval for
+            # overlaps and delete those to avoid the trigger (note that
+            # this does not actually delete anything if dry_run is true)
+            push @deleted_set, @{ $self->_clobber_intervals( $tempintvl ) } if $self->clobber;
+
             my $int = App::Dochazka::REST::Model::Interval->spawn(
                           eid => $self->emp_obj->eid,
                           aid => $self->act_obj->aid,
@@ -727,10 +754,10 @@ sub commit {
             # INSERT only if not dry run
             if ( ! $self->dry_run ) {
                 $status = $int->insert( $self->context );
-                if( $status->not_ok ) {
+                if ( $status->not_ok ) {
                     push @fail_set, {
                         interval => $int,
-                        status => $status,
+                        status => $status->expurgate,
                     };
                     $not_ok_count += 1;
                     next TEMPINTVL_LOOP;
@@ -744,11 +771,7 @@ sub commit {
     }
 
     $count = $ok_count + $not_ok_count;
-    if ( $count ) {
-        return $CELL->status_ok( 
-            'DISPATCH_FILLUP_INTERVALS_CREATED', 
-            args => [ $count ],
-            payload => {
+    my $pl = {
                 "success" => {
                     count => $ok_count,
                     intervals => \@result_set, 
@@ -757,13 +780,58 @@ sub commit {
                     count => $not_ok_count,
                     intervals => \@fail_set,
                 },
-            },
+            };
+    $pl->{"clobbered"} = {
+        count => scalar( @deleted_set ),
+        intervals => \@deleted_set
+    } if $self->clobber;
+    if ( $count ) {
+        return $CELL->status_ok( 
+            'DISPATCH_FILLUP_INTERVALS_CREATED', 
+            args => [ $count ],
+            payload => $pl,
             count => $count, 
         );
     }
     return $CELL->status_notice( 'DISPATCH_FILLUP_NO_INTERVALS_CREATED' );
 }
 
+# given a tempintvl object, clobbers any intervals that conflict with it,
+# logs errors and returns reference to set of deleted interval objects
+sub _clobber_intervals {
+    my ( $self, $tempintvl ) = @_;
+
+    my @clobbered_intervals = ();
+
+    my $status = fetch_intervals_by_eid_and_tsrange_inclusive(
+        $self->context->{'dbix_conn'},
+        $self->emp_obj->eid,
+        $tempintvl->intvl,
+    );
+    if ( $status->ok and $status->code eq 'DISPATCH_RECORDS_FOUND' ) {
+        foreach my $int ( @{ $status->payload } ) {
+            if ( $self->dry_run ) {
+                push @clobbered_intervals, $int;
+            } else {
+                my $saved_int = $int->clone;
+                my $status = $int->delete( $self->context );
+                if ( $status->ok ) {
+                    push @clobbered_intervals, $saved_int;
+                } else {
+                    $log->error( "Could not delete interval " . $int->intvl .
+                                 " due to " .  $status->text );
+                }
+            }
+        }
+    } elsif ( $status->level eq 'NOTICE' and $status->code eq 'DISPATCH_NO_RECORDS_FOUND' ) {
+        $log->debug( "Interval " . $tempintvl->intvl . 
+                     " does not overlap with any existing intervals" );
+    } else {
+        $log->crit( "FILLUP COMMIT: " . $status->text );
+    }
+    return \@clobbered_intervals;
+}
+
 
 =head2 DESTROY
 
@@ -774,7 +842,7 @@ Returns a status object.
 
 sub DESTROY {
     my $self = shift;
-    $log->debug( "Entering " . __PACKAGE__ . "::DESTROY from " . Dumper( \caller ) );
+    $log->debug( "Entering " . __PACKAGE__ . "::DESTROY with arguments " .  join( ' ', @_ ) );
 
     $log->notice( "GLOBAL DESTRUCTION" ) if ${^GLOBAL_PHASE} eq 'DESTRUCT';
 
@@ -797,7 +865,8 @@ sub DESTROY {
         $status = $CELL->status_err( 'DOCHAZKA_DBI_ERR', args => [ $_ ] );
     };
     $log->notice( "Fillup destructor says " . $status->level . ": " . $status->text );
-    return;
+    return $status if $status;
+    return $CELL->status_ok;
 }
 
 
